/*
 * Copyright (C) 2011 Wireless Network and Multimedia Laboratory, NCU, Taiwan
 * 
 * You can reference http://wmlab.csie.ncu.edu.tw
 * 
 * This class is used to interact with remote device. We use this as a user interface to help us debug.
 * 
 * 
 * @author Fiona
 * @version 0.0.1
 *
 */

package com.newwmlab.bluetoothconn;


import android.app.Activity;
import android.app.AlertDialog;
import android.app.Dialog;
import android.bluetooth.BluetoothAdapter;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.preference.PreferenceManager;
import android.util.Log;
import android.view.KeyEvent;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.Window;
import android.view.inputmethod.EditorInfo;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;
import com.newwmlab.bluetoothconn.R;

public class BluetoothConn extends Activity {
    // Debugging
    private static final String TAG = "BluetoothConn";
    private static final boolean D = true; 
    
    private static String versionNo = "1.0.0";

    // Key names received from the BluetoothChatService Handler
    public static final String DEVICE_NAME = "device_name";
    public static final String TOAST = "toast";

    public static final String NOTIFY_UI = "NOTIFY_UI";
    public static final String INCOMING_MSG = "INCOMING_MSG";
    public static final String OUTGOING_MSG = "OUTGOING_MSG";
    public static final String ALERT_MSG = "ALERT_MSG";
    public static final String KEY_ECHO_PREF = "KEY_ECHO_PREF";
    public static final String KEY_SHOW_RX_PREF = "KEY_SHOW_RX_PREF";
    public static final String DEVICE_ADDRESS = "device_address";
    public static final String DISCONNECT_DEVICE_ADDRESS = "disconnected_device_address";
   
    
    // Intent request codes
    private static final int REQUEST_CONNECT_DEVICE = 1;
    private static final int REQUEST_ENABLE_BT = 2;
    private static final int REQUEST_PREF_SETTING = 3;
    private static final int REQUEST_DISCONNECT_DEVICE =  4;
    private static final int REQUEST_ABOUT_INFO = 5;
    
    private static final int DIALOG_ABOUT_INFO = 6;
    private static final int FILE_DIALOG_REQUEST = 7;

    // Layout Views
    private TextView mTitle;
    private ListView mConversationView;
    //private EditText mOutEditText;
    private Button mSendButton;

    // Name of the connected device
    // Array adapter for the conversation thread
    private ArrayAdapter<String> mConversationArrayAdapter;
    // String buffer for outgoing messages
    private StringBuffer mOutStringBuffer;
    // Local Bluetooth adapter
    private BluetoothAdapter mBluetoothAdapter = null;
    private NotificationReceiver mNotiReceiver;
    private boolean mMonitor = false;

    public static final String TEST_ACTION = "TEST_ACTION";    


    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        if(D) Log.e(TAG, "+++ ON CREATE +++");

        // Set up the window layout
        requestWindowFeature(Window.FEATURE_CUSTOM_TITLE);
        setContentView(R.layout.main);
        getWindow().setFeatureInt(Window.FEATURE_CUSTOM_TITLE, R.layout.custom_title);//redirect to layout\custom_title.xml

        // Set up the custom title
        mTitle = (TextView) findViewById(R.id.title_left_text);
        mTitle.setText(R.string.app_name);
       // mTitle = (TextView) findViewById(R.id.title_right_text);
       // mTitle.setText("monitor: " + mMonitor);
		IntentFilter mFilter01, mFilter02;
        mFilter01 = new IntentFilter(INCOMING_MSG);
        mFilter02 = new IntentFilter(OUTGOING_MSG);
        mNotiReceiver = new NotificationReceiver();
        registerReceiver(mNotiReceiver, mFilter01);
        registerReceiver(mNotiReceiver, mFilter02);
        registerReceiver(mNotiReceiver, new IntentFilter(ALERT_MSG));
        registerReceiver(mNotiReceiver, new IntentFilter(BluetoothConnController.GET_SERIVICE_STATUS_EVENT));
   //     registerReceiver(mNotiReceiver, new IntentFilter(BluetoothDevice.ACTION_ACL_DISCONNECTED));

        // Get local Bluetooth adapter
        mBluetoothAdapter = BluetoothAdapter.getDefaultAdapter();

        // If the adapter is null, then Bluetooth is not supported
        if (mBluetoothAdapter == null) {
            Toast.makeText(this, "Bluetooth is not available", Toast.LENGTH_LONG).show();
            finish();
            return;
        }
        
 /*       //aa
        String str = "test 12345";
        try {
        	
        	FileOutputStream fos = openFileOutput("test_file2.txt", Context.MODE_WORLD_READABLE);
        	fos.write(str.getBytes());
        	fos.close();
        	Log.d(TAG, "save file OK");
        	FileInputStream rfos = openFileInput("test_file2.txt");
        	byte[] str2 = new byte[20];
        	int a = rfos.read(str2);
        	str2[a] = 0x00;
        	Log.d(TAG, "read file " + a +  new String(str2,0, a));
  
        	
        }
        catch (Exception e) {
        	Log.e(TAG, "save file fail", e);
        }
      	boolean mExternalStorageAvailable = false;
    	boolean mExternalStorageWriteable = false;
    	String state = Environment.getExternalStorageState();
    	if (Environment.MEDIA_MOUNTED.equals(state)) {    
    		// We can read and write the media    
    		mExternalStorageAvailable = mExternalStorageWriteable = true;
    		
    	} 
    	else if (Environment.MEDIA_MOUNTED_READ_ONLY.equals(state)) {    
    		// We can only read the media    
    		mExternalStorageAvailable = true;   
    		mExternalStorageWriteable = false;
    	} 
    	Log.d(TAG, "available: " + mExternalStorageAvailable +",  writeable: " + mExternalStorageWriteable);
    	*/
 /*       File root = Environment.getExternalStorageDirectory();
        try {
        	FileOutputStream fos_e = new FileOutputStream(root + "/" + "ext_test3.txt", true);
        	fos_e.write(str.getBytes());
        	fos_e.write(str.getBytes());
        	fos_e.close();
        }
        catch (Exception e) {
        	Log.e(TAG, "external storage save fail", e);
        }*/
/*        
  		try {
        	File f = new File(root + "/" + "ext_test1.txt");
        	f.delete();
        }
        catch(Exception e) {
        	
        }*/
        ///////////aa

    }

    @Override
    public void onStart() {
        super.onStart();
        if(D) Log.e(TAG, "++ ON START ++");

        // If BT is not on, request that it be enabled.
        // setupChat() will then be called during onActivityResult
        if (!mBluetoothAdapter.isEnabled()) {
            Intent enableIntent = new Intent(BluetoothAdapter.ACTION_REQUEST_ENABLE);
            startActivityForResult(enableIntent, REQUEST_ENABLE_BT);
        // Otherwise, setup the chat session
        } else {
        	setupChat();        	
        }
    }

    @Override
    public synchronized void onResume() {
        super.onResume();
        if(D) Log.e(TAG, "+ ON RESUME +");

    }

    private void setupChat() {
        Log.d(TAG, "setupChat()");

        // Initialize the array adapter for the conversation thread
        mConversationArrayAdapter = new ArrayAdapter<String>(this, R.layout.message);//redirect to layout\message.xml
        mConversationView = (ListView) findViewById(R.id.in);
        mConversationView.setAdapter(mConversationArrayAdapter);

        // Initialize the compose field with a listener for the return key
        //mOutEditText = (EditText) findViewById(R.id.edit_text_out);
        //mOutEditText.setOnEditorActionListener(mWriteListener);

        // Initialize the send button with a listener that for click events
        mSendButton = (Button) findViewById(R.id.send);
        mSendButton.setOnClickListener(new OnClickListener() {
  			@Override
			public void onClick(View arg0) {
	               TextView view = (TextView) findViewById(R.id.editText);
	                String message = view.getText().toString();
	                if (message.length() == 0) {
	                	return;
	                }
	                Intent i = new Intent(BluetoothConnController.SEND_MSG_FROM_BT_ACTION);
			    	i.putExtra("MESSAGE", message);
			    	i.putExtra("MODE", BluetoothConnController.MSG_MODE_SEND_STRING);
			    	sendBroadcast(i);
	                mOutStringBuffer.setLength(0);
	                view.setText("");
			}
        });
        

        // Initialize the BluetoothConnController to perform bluetooth connections
        Intent startIntent = new Intent(BluetoothConn.this, BluetoothConnController.class);
    	startService(startIntent);

    	//get monitor status, ��"�^�W�@��", activity �| destroy, �� sevice �|�~��s�b, �y�� status ���P�B
        Intent i = new Intent(BluetoothConnController.GET_SERIVICE_STATUS_ACTION);
        sendBroadcast(i);
        // Initialize the buffer for outgoing messages
        mOutStringBuffer = new StringBuffer("");
    }

    @Override
    public synchronized void onPause() {
        super.onPause();
        if(D) Log.e(TAG, "- ON PAUSE -");
    }

    @Override
    public void onStop() {
        super.onStop();
        if(D) Log.e(TAG, "-- ON STOP --");
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        // Stop the Bluetooth chat services
   //     if (mConnService != null) mConnService.terminated();
   //     unregisterReceiver(mBtBroadcastReceiver);
        if(D) Log.e(TAG, "--- ON DESTROY ---");
        unregisterReceiver(mNotiReceiver);
    }

    private void ensureDiscoverable() {
        if(D) Log.d(TAG, "ensure discoverable");
        if (mBluetoothAdapter.getScanMode() !=
            BluetoothAdapter.SCAN_MODE_CONNECTABLE_DISCOVERABLE) {
            Intent discoverableIntent = new Intent(BluetoothAdapter.ACTION_REQUEST_DISCOVERABLE);
            discoverableIntent.putExtra(BluetoothAdapter.EXTRA_DISCOVERABLE_DURATION, 300);
            startActivity(discoverableIntent);
        }
    }

    // The action listener for the EditText widget, to listen for the return key
    private TextView.OnEditorActionListener mWriteListener =
        new TextView.OnEditorActionListener() {
        public boolean onEditorAction(TextView view, int actionId, KeyEvent event) {
            // If the action is a key-up event on the return key, send the message
            if (actionId == EditorInfo.IME_NULL && event.getAction() == KeyEvent.ACTION_UP) {
                String message = view.getText().toString();
                if (message.length() == 0){
                	return true;
                }
                Intent i = new Intent(BluetoothConnController.SEND_MSG_FROM_BT_ACTION);
		    	i.putExtra("MESSAGE", message);
		    	i.putExtra("MODE", BluetoothConnController.MSG_MODE_SEND_STRING);
		    	sendBroadcast(i);	    	
               // sendMessage(message);
                mOutStringBuffer.setLength(0);
                //mOutEditText.setText(mOutStringBuffer);
            }
            if(D) Log.i(TAG, "END onEditorAction");
            return true;
        }
    };

    public void onActivityResult(int requestCode, int resultCode, Intent data) {
        if(D) Log.d(TAG, "onActivityResult: requestCode="+requestCode+ ", resultCode=" + resultCode);
        switch (requestCode) {
        case REQUEST_CONNECT_DEVICE:
            // When DeviceListActivity returns with a device to connect
            if (resultCode == Activity.RESULT_OK) {
                // Get the device MAC address
                String address = data.getExtras()
                                     .getString(DeviceListActivity.EXTRA_DEVICE_ADDRESS);
                // Get the BLuetoothDevice object
             //   BluetoothDevice device = mBluetoothAdapter.getRemoteDevice(address);
                // Attempt to connect to the device
                //mConnService.connectTo(device);
                Intent i = new Intent(BluetoothConnController.CONNECT_REQUEST_ACTION);
                i.putExtra(DEVICE_ADDRESS, address);
                sendBroadcast(i);
            }
            break;
        case REQUEST_DISCONNECT_DEVICE:
        	if (resultCode == Activity.RESULT_OK) {
        		String address = data.getExtras()
                    .getString(ConnectedDeviceListActivity.EXTRA_CONNECTED_ADDRESS);
        		Intent i = new Intent(BluetoothConnController.DISCONNECT_REQUEST_ACTION);
        		i.putExtra(DISCONNECT_DEVICE_ADDRESS, address);
        		sendBroadcast(i);
        		
        	}
        	break;
        case REQUEST_ENABLE_BT:
            // When the request to enable Bluetooth returns
            if (resultCode == Activity.RESULT_OK) {
                // Bluetooth is now enabled, so set up a chat session
                setupChat();
            } else {
                // User did not enable Bluetooth or an error occured
                Log.d(TAG, "BT not enabled");
                Toast.makeText(this, R.string.bt_not_enabled_leaving, Toast.LENGTH_SHORT).show();
                finish();
            }
            break;
            
        case REQUEST_PREF_SETTING:
        // When the request to enable Bluetooth returns
        	if (resultCode == Activity.RESULT_OK) {
        		SharedPreferences settings = PreferenceManager.getDefaultSharedPreferences(this);
        		boolean isEcho = settings.getBoolean(KEY_ECHO_PREF, false);
        		Log.e(TAG, "[onActivityResult] Is Echo = "+isEcho);
        	//	Intent settingIntent = new Intent(ServiceController.CHANGE_LOCAL_SETTING_ACTION);
        	//	settingIntent.putExtra(ServiceModel.KEY_LOCAL_SETTING, ServiceModel.TYPE_SIGNAL_LEVEL_SETTING);
        	//	sendBroadcast(settingIntent);
        	}
            break;
            
        case REQUEST_ABOUT_INFO:
        	if (resultCode == Activity.RESULT_OK) {
        		Log.e(TAG, "[onActivityResult] Current version is = "+versionNo);
        	}
        	break;
        case FILE_DIALOG_REQUEST:
        	if (resultCode == Activity.RESULT_OK) {
            	Log.d(TAG, "receive FILE_DIALOG_REQUEST result: " + resultCode);
            	String file = data.getExtras().getString(FileDialog.RESULT_PATH);
            	Intent i = new Intent(BluetoothConnController.SEND_MSG_FROM_BT_ACTION);
            	i.putExtra("MESSAGE", file.toString());
    	    	i.putExtra("MODE", BluetoothConnController.MSG_MODE_SEND_FILE);
    	    	sendBroadcast(i);           		
        	}

        	break;
    
        }
    }

    @Override
	protected Dialog onCreateDialog(int id) {
		switch(id){
		case DIALOG_ABOUT_INFO:
			return new AlertDialog.Builder(BluetoothConn.this)
			.setIcon(R.drawable.ic_menu_help)
			.setTitle(R.string.about_title)
			.setMessage(R.string.about_content)
			.setPositiveButton(R.string.about_dialog_ok, new DialogInterface.OnClickListener(){

				
				public void onClick(DialogInterface dialog, int button) {
					
				}
			})
			.create();
			//.setTitle(R.string.about)
			
		}
		return null;
	}

	@Override
    public boolean onCreateOptionsMenu(Menu menu) {
        MenuInflater inflater = getMenuInflater();
        inflater.inflate(R.menu.option_menu, menu);
        return true;
    }

   
	/* (non-Javadoc)
	 * @see android.app.Activity#onPrepareOptionsMenu(android.view.Menu)
	 */
	@Override
	public boolean onPrepareOptionsMenu(Menu menu) {
		// TODO Auto-generated method stub
		Log.d(TAG, "onPrepareOptionsMenu");
		return super.onPrepareOptionsMenu(menu);
	}

	/* (non-Javadoc)
	 * @see android.app.Activity#onMenuOpened(int, android.view.Menu)
	 */
	@Override
	public boolean onMenuOpened(int featureId, Menu menu) {
		// TODO Auto-generated method stub
		Log.d(TAG, "onMenuOpened");
		boolean status = super.onMenuOpened(featureId, menu);
		MenuItem item = (MenuItem) menu.findItem(R.id.monitor);			
		item.setTitle(mMonitor?"Stop Monitir":"Start Monitor");	
		return status;
	}

	@Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
        case R.id.scan:
            // Launch the DeviceListActivity to see devices and do scan
            Intent serverIntent = new Intent(this, DeviceListActivity.class);
            startActivityForResult(serverIntent, REQUEST_CONNECT_DEVICE);
            break;
        case R.id.inquiry:
            // Launch the DeviceListActivity to see devices and do scan
        	Intent settintIntent = new Intent(android.provider.Settings.ACTION_BLUETOOTH_SETTINGS);
        	startActivity(settintIntent);
            break;
        case R.id.discoverable:
            // Ensure this device is discoverable by others
            ensureDiscoverable();
            break;
        case R.id.disconnect:
        	Log.i(TAG, "[onOptionsItemSelected] R.id.disconnect");
        	Intent disconnectIntent = new Intent(this, ConnectedDeviceListActivity.class);
        	startActivityForResult(disconnectIntent, REQUEST_DISCONNECT_DEVICE);
        	break;  
            
        case R.id.setting:
        	Intent settingIntent = new Intent(this, SettingPreferencActivity.class);
			startActivityForResult(settingIntent,REQUEST_PREF_SETTING );
			break; 
        case R.id.monitor:
        	if (mMonitor == false){
 
                	mMonitor = true;
                	item.setTitle("Stop Monitor");	
        	} 
        	else {
        		item.setTitle("Start Monitor");
        		mMonitor = false;
        	} 
        	//Log.d(TAG,"monitor "+ mMonitor);
        	Intent i = new Intent(BluetoothConnController.START_MONITOR_ACTION);
            i.putExtra(BluetoothConnController.MONITOR_STATUS, mMonitor);
            sendBroadcast(i);
            mTitle = (TextView) findViewById(R.id.title_right_text);
			mTitle.setText("monitor: " + mMonitor);
			break;
        case R.id.file_dialog:
        	Intent fileDialog = new Intent(this, FileDialog.class);
        	startActivityForResult(fileDialog, FILE_DIALOG_REQUEST);
        	break;
        case R.id.about:
        	//Intent aboutIntent = new Intent(this, AboutInfoActivity.class);
			//startActivityForResult(aboutIntent,REQUEST_PREF_SETTING );
        	showDialog(DIALOG_ABOUT_INFO);
        	break;
        default:
        	return false;
        }
        return true;
    }
    public class NotificationReceiver extends BroadcastReceiver {

		@Override
		public void onReceive(Context context, Intent intent) {			
			String action  = intent.getAction();
			if(D) Log.e(TAG, "** ON RECEIVE **" + action);
			if (action.equals(INCOMING_MSG)){
				String displayMsg = intent.getExtras().getString("STR");
	      		SharedPreferences settings = PreferenceManager.getDefaultSharedPreferences(BluetoothConn.this);
        		boolean isSet = settings.getBoolean(KEY_SHOW_RX_PREF, false);
        		if (isSet) {
    				mConversationArrayAdapter.add(displayMsg);
				}
				int num = intent.getExtras().getInt("COUNTER");
				mTitle = (TextView) findViewById(R.id.rxLabel);
				mTitle.setText("rx: " + num);
			} 
			else if (action.equals(OUTGOING_MSG)) {
				String displayMsg = intent.getExtras().getString("STR");
				mConversationArrayAdapter.add(displayMsg);
				int num = intent.getExtras().getInt("COUNTER");
				mTitle = (TextView) findViewById(R.id.txLabel);
				mTitle.setText("tx: " + num);
			}
			else if (action.equals(ALERT_MSG)) {
				String displayMsg = intent.getExtras().getString("STR");
				mConversationArrayAdapter.add(displayMsg);
				Log.d(TAG, displayMsg + "counter = " + mConversationArrayAdapter.getCount());
				if (intent.getExtras().getInt("COUNTER") > 0){
					AlertDialog.Builder builder = new AlertDialog.Builder(BluetoothConn.this);
					builder.setIcon(android.R.drawable.stat_notify_error)
					.setTitle("Error")
					.setMessage(displayMsg)
					.setPositiveButton("OK", new DialogInterface.OnClickListener(){
						
						public void onClick(DialogInterface dialog, int button) {					
						}					
					})
					.create();
					AlertDialog alert = builder.create();
					alert.show();
				}				
			}
			else if (action.equals(BluetoothConnController.GET_SERIVICE_STATUS_EVENT)) {
				int num = intent.getExtras().getInt(BluetoothConnController.TX_BYTES);
				mTitle = (TextView) findViewById(R.id.txLabel);
				mTitle.setText("tx: " + num);
				num = intent.getExtras().getInt(BluetoothConnController.RX_BYTES);
				mTitle = (TextView) findViewById(R.id.rxLabel);
				mTitle.setText("rx: " + num);
				mMonitor = intent.getExtras().getBoolean(BluetoothConnController.MONITOR_STATUS, mMonitor);
				mTitle = (TextView) findViewById(R.id.title_right_text);
				mTitle.setText("monitor: " + mMonitor);
				Log.d(TAG, mTitle.getText().toString());
			}
/*			else if (action.equals(BluetoothDevice.ACTION_ACL_DISCONNECTED)) {
				Toast.makeText(BluetoothConn.this, "Bluetooth connection is disconnected!", Toast.LENGTH_LONG).show();

			}*/
			else{
				Log.e(TAG, "another action: " + action);
			}
		}
    	
    }
}